angular.module('beamng.apps')
.directive('simplePerfTimersPlus', ['$log', 'StreamsManager', function ($log, StreamsManager) {
  return {
    template:
        '<div style="width:100%; height:100%; background-color: rgba(255,255,255,0.9)" layout="column">' +
          '<div flex style="overflow-y:scroll">' +
            '<table style="width:100%">' +
            '<thead><tr><th><b>Type</b></th> <th><b>Start</b></th> <th><b>End</b></th> <th><b>Time</b></th> <th><b>Distance</b></th> </tr></thead>' +
            '<tbody style="text-align:center"><tr ng-repeat="row in timers">' +
             '<td>{{ ::row.type }}</td> <td>{{ ::row.start }}</td> <td>{{ ::row.stop }}</td> <td>{{ ::row.time }}</td> <td>{{ ::row.distance }}</td>' +
            '</tr></tbody>' +
          '</table>' +
          '</div>' +
          '<div style="min-height:1em;">{{ msg.test }}<br>{{ msg.abort }}</div>' +
        '</div>',
    replace: true,
    restrict: 'EA',
    link: function (scope, element, attrs) {
      var streamsList = ['electrics', 'sensors', 'stats'];
      StreamsManager.add(streamsList);
      scope.$on('$destroy', function () {
        $log.debug('<simple-perf-timers-plus> destroyed');
        StreamsManager.remove(streamsList);
      });

      scope.timers = [];

      var unitSystems = {
        metric: {
          units: 'km/h',
          timerSpeedLimit: 100,
          multiplier: 3.6
        },

        imperial: {
          units: 'MPH',
          timerSpeedLimit: 60,
          multiplier: 2.23693629
        }
      };

      var unitSystem = unitSystems['metric'];

      scope.$on('SettingsChanged', function (event, data) {
        unitSystem = unitSystems[data.values.uiUnits];
      });

      var speedMargin     = 0.5
        , wheelTimerState = 0
        , wheelTimer      = 0
        , prevTime = 0
        , curTime  = 0
      ;

      // var prevTime, curTime;
      var startPos, startVelo, stopVelo, aborted, testing;
      scope.msg = {
        test: '',
        abort: ''
      };

      scope.$on('streamsUpdate', function (event, streams) {
        var throttle = streams.electrics.throttle
          , brake = streams.electrics.brake
          , position = streams.sensors.position
          , airspeed = streams.electrics.airspeed * unitSystem.multiplier
        ;

        prevTime = curTime;
        curTime = performance.now();
        var dt = (curTime - prevTime) / 1000;

        testing = ' ';
        aborted = ' ';

        // Speed testing.
        if (throttle > 0.2 && airspeed > speedMargin && airspeed < 3*speedMargin && wheelTimerState === 0 ) {
          wheelTimerState = 1;
		  doOnce = 0;
          wheelTimer = 0;
          startPos = position;
          startVelo = airspeed;
          aborted = ' ';
          testing = ' ';
        } else if (wheelTimerState == 1) {
          wheelTimer += dt;
		  
		  
		  if (airspeed >= unitSystem.timerSpeedLimit) {
            var stopPos = position;
            stopVelo = airspeed;
            var distance = Math.sqrt(Math.pow((stopPos.x - startPos.x), 2) + Math.pow((stopPos.y - startPos.y), 2) + Math.pow((stopPos.z - startPos.z), 2));
			//Quarter mile stuff
			var ft = distance * 3.281;
            if (ft >= 1320) {
				scope.timers.unshift({
				  type: 'Qtr mile',
				  start: Math.floor(startVelo) + ' ' + unitSystem.units,
				  stop:  Math.floor(stopVelo) + ' ' + unitSystem.units,
				  time: wheelTimer.toFixed(2) + ' s',
				  distance: '1320 ft',
				});
				wheelTimerState = 0;
				testing = ' ';
			}
			//protect 0 - 60/100 timer
			if (doOnce == 0) {
				scope.timers.unshift({
				  type: 'Speed',
				  start: Math.floor(startVelo) + ' ' + unitSystem.units,
				  stop:  Math.floor(stopVelo) + ' ' + unitSystem.units,
				  time: wheelTimer.toFixed(2) + ' s',
				  distance: distance.toFixed(2) + ' ft'
				});
				doOnce = 1;
			}
          }
		 //raise throttle threshold slightly to allow for wheelspin control
          if (throttle < 0.2) {
            aborted = 'Aborted: Throttle < 0.2';
            wheelTimerState = 0;
          }
        }
        if (wheelTimerState == 1) {
          testing = 'Speed testing... ' + airspeed.toFixed(2) + ' ' + unitSystem.units + ', ' + wheelTimer.toFixed(2) + ' s';
        }

        // brake testing
        if (brake > 0.5 && airspeed > 10 - speedMargin && wheelTimerState === 0) {
          wheelTimerState = 3;
          wheelTimer = 0;
          startPos = position;
          startVelo = airspeed;
          aborted = ' ';
          testing = ' ';
        } else if (wheelTimerState == 3) {
          wheelTimer += dt;

          if (airspeed < speedMargin) {
            var stopPos2 = position;
            stopVelo = airspeed;
            var distance2 = Math.sqrt(Math.pow((stopPos2.x - startPos.x), 2) + Math.pow((stopPos2.y - startPos.y), 2)  + Math.pow((stopPos2.z - startPos.z), 2));
            wheelTimerState = 0;
            testing = ' ';
            scope.timers.unshift({
              type: 'Brake',
              start: Math.floor(startVelo) + ' ' + unitSystem.units,
              stop:  Math.floor(stopVelo) + ' ' + unitSystem.units,
              time: wheelTimer.toFixed(2) + ' s',
              distance: distance2.toFixed(2) + ' m'
            });
          }

          if (brake < 0.5) {
            console.log('the brake is ', brake);
            aborted = 'Aborted: Brake < 0.5';
            wheelTimerState = 0;
          }
        }

        if (wheelTimerState == 3) {
          testing = 'Brake testing... ' + airspeed.toFixed(2) + ' ' + unitSystem.units + ', ' + wheelTimer.toFixed(2) + ' s';
        }

        scope.$evalAsync(function () {
          scope.msg = {test: testing, abort: aborted};
        });
      });

      scope.$on('$destroy', function () {
        $log.debug('<simple-perf-timers-plus destroyed');
        StreamsManager.remove(streamsList);
      });
    }
  }
}])
